@@ -24,14 +24,14 @@ class ReadPoller(object):
     BlockingConnection._handle_read() method.
 
     """
-    POLL_TIMEOUT = 0.01
+    POLL_TIMEOUT = 10
 
     def __init__(self, fd, poll_timeout=POLL_TIMEOUT):
         """Create a new instance of the ReadPoller which wraps poll and select
         to determine if the socket has data to read on it.
 
         :param int fd: The file descriptor for the socket
-        :param float poll_timeout: How long to wait for events
+        :param float poll_timeout: How long to wait for events (milliseconds)
 
         """
         self.fd = fd
@@ -42,6 +42,7 @@ class ReadPoller(object):
             self.poller.register(self.fd, self.poll_events)
         else:
             self.poller = None
+            self.poll_timeout = float(poll_timeout) / 1000
 
     def ready(self):
         """Check to see if the socket has data to read.
@@ -74,6 +75,37 @@ class BlockingConnection(base_connection.BaseConnection):
     SOCKET_TIMEOUT_CLOSE_THRESHOLD = 3
     SOCKET_TIMEOUT_MESSAGE = "Timeout exceeded, disconnected"
 
+    def __init__(self, parameters=None):
+        """Create a new instance of the Connection object.
+
+        :param pika.connection.Parameters parameters: Connection parameters
+        :raises: RuntimeError
+
+        """
+        super(BlockingConnection, self).__init__(parameters, None, False)
+
+    def add_on_close_callback(self, callback_method_unused):
+        """This is not supported in BlockingConnection. When a connection is
+        closed in BlockingConnection, a ConnectionClosed exception will be
+        raised.
+
+        :param method callback_method_unused: Unused
+        :raises: NotImplementedError
+
+        """
+        raise NotImplementedError('Blocking connection will raise '
+                                  'ConnectionClosed exception')
+
+    def add_on_open_callback(self, callback_method_unused):
+        """This method is not supported in BlockingConnection.
+
+        :param method callback_method_unused: Unused
+        :raises: NotImplementedError
+
+        """
+        raise NotImplementedError('Connection callbacks not supported in '
+                                  'BlockingConnection')
+
     def add_timeout(self, deadline, callback):
         """Add the callback to the IOLoop timer to fire after deadline
         seconds.
@@ -83,9 +115,9 @@ class BlockingConnection(base_connection.BaseConnection):
         :rtype: str
 
         """
-        timeout_id = '%.8f' % time.time()
-        self._timeouts[timeout_id] = {'deadline': deadline + time.time(),
-                                      'method': callback}
+        value = {'deadline': time.time() + deadline, 'callback': callback}
+        timeout_id = hash(frozenset(value))
+        self._timeouts[timeout_id] = value
         return timeout_id
 
     def channel(self, channel_number=None):
@@ -111,6 +143,7 @@ class BlockingConnection(base_connection.BaseConnection):
         :param str reply_text: The text reason for the close
 
         """
+        self._set_connection_state(self.CONNECTION_CLOSING)
         self._remove_connection_callbacks()
         super(BlockingConnection, self).close(reply_code, reply_text)
         self.process_data_events()
@@ -118,7 +151,9 @@ class BlockingConnection(base_connection.BaseConnection):
 
     def disconnect(self):
         """Disconnect from the socket"""
-        self.socket.close()
+        self._set_connection_state(self.CONNECTION_CLOSED)
+        if self.socket:
+           self.socket.close()
 
     def process_data_events(self):
         """Will make sure that data events are processed. Your app can
@@ -189,6 +224,7 @@ class BlockingConnection(base_connection.BaseConnection):
 
         LOGGER.debug('Setting socket timeout to %s', self.params.socket_timeout)
         self.socket.settimeout(self.params.socket_timeout)
+        self._set_connection_state(self.CONNECTION_OPEN)
         LOGGER.info('Adapter connected')
 
     def _adapter_disconnect(self):
@@ -202,8 +238,8 @@ class BlockingConnection(base_connection.BaseConnection):
         :param dict timeout_value: The configuration for the timeout
 
         """
-        LOGGER.debug('Invoking scheduled call of %s', timeout_value['method'])
-        timeout_value['method']()
+        LOGGER.debug('Invoking scheduled call of %s', timeout_value['callback'])
+        timeout_value['callback']()
 
     def _deadline_passed(self, timeout_id):
         """Returns True if the deadline has passed for the specified timeout_id.
@@ -263,7 +299,7 @@ class BlockingConnection(base_connection.BaseConnection):
 
         :param pika.frame.Method: The Connection.Close frame
         :param bool from_adapter: Called by the connection adapter
-        :raises: AMQPConnectionError
+        :raises: pika.exceptions.ConnectionClosed
 
         """
         LOGGER.info('on_connection_closed: %r, %r', method_frame, from_adapter)
@@ -281,7 +317,7 @@ class BlockingConnection(base_connection.BaseConnection):
             self._channels[channel]._on_close(method_frame)
         self._remove_connection_callbacks()
         if self.closing[0] != 200:
-            raise exceptions.AMQPConnectionError(*self.closing)
+            raise exceptions.ConnectionClosed(*self.closing)
 
     def _send_frame(self, frame_value):
         """This appends the fully generated frame to send to the broker to the
@@ -504,6 +540,7 @@ class BlockingChannel(channel.Channel):
         :return int: The number of messages requeued by Basic.Nack
 
         """
+        messages = 0
         self.basic_cancel(self._generator)
         if self._generator_messages:
             # Get the last item
@@ -748,7 +785,7 @@ class BlockingChannel(channel.Channel):
         replies = [spec.Queue.PurgeOk] if nowait is False else []
         return self._rpc(spec.Queue.Purge(0, queue, nowait), None, replies)
 
-    def queue_unbind(self, queue='', exchange=None, routing_key='',
+    def queue_unbind(self, queue='', exchange=None, routing_key=None,
                      arguments=None):
         """Unbind a queue from an exchange.
 
@@ -761,6 +798,8 @@ class BlockingChannel(channel.Channel):
         :param dict arguments: Custom key/value pair arguments for the binding
 
         """
+        if routing_key is None:
+            routing_key = queue
         return self._rpc(spec.Queue.Unbind(0, queue, exchange, routing_key,
                                            arguments or dict()), None,
                          [spec.Queue.UnbindOk])
@@ -867,8 +906,11 @@ class BlockingChannel(channel.Channel):
             self._send_method(spec.Channel.CloseOk(), None, False)
         self._set_state(self.CLOSED)
         self._cleanup()
-        raise exceptions.ChannelClosed(method_frame.method.reply_code,
-                                       method_frame.method.reply_text)
+        if method_frame is None:
+            raise exceptions.ChannelClosed(0, 'Not specified')
+        else:
+            raise exceptions.ChannelClosed(method_frame.method.reply_code,
+                                           method_frame.method.reply_text)
 
     def _on_openok(self, method_frame):
         """Open the channel by sending the RPC command and remove the reply
@@ -987,6 +1029,7 @@ class BlockingChannel(channel.Channel):
         self._rpc(spec.Channel.Close(self._reply_code, self._reply_text, 0, 0),
                   None,
                   [spec.Channel.CloseOk])
+        self._set_state(self.CLOSED)
         self._cleanup()
 
     def _validate_acceptable_replies(self, acceptable_replies):
