@@ -1,4 +1,5 @@
 """Core connection objects"""
+import ast
 import logging
 import platform
 import urllib
@@ -465,9 +466,10 @@ class URLParameters(Parameters):
             self._validate_socket_timeout(values['socket_timeout'])):
             self.socket_timeout = values['socket_timeout']
 
-        if ('ssl_options' in values and
-            self._validate_ssl_options(values['ssl_options'])):
-            self.ssl_options = values['ssl_options']
+        if 'ssl_options' in values:
+            options = ast.literal_eval(values['ssl_options'])
+            if self._validate_ssl_options(options):
+                self.ssl_options = options
 
 
 class Connection(object):
@@ -582,17 +584,18 @@ class Connection(object):
             LOGGER.warning("Invoked while closing or closed")
             return
 
+        if self._has_open_channels:
+            self._close_channels(reply_code, reply_text)
+
         # Set our connection state
         self._set_connection_state(self.CONNECTION_CLOSING)
         LOGGER.info("Closing connection (%s): %s", reply_code, reply_text)
         self.closing = reply_code, reply_text
 
-        # If channels are open, _on_close_ready will be called when they close
-        if self._has_open_channels:
-            return self._close_channels(reply_code, reply_text)
-        else:
+        if not self._has_open_channels:
+            # if there are open channels then _on_close_ready will finally be
+            # called in _on_channel_closeok once all channels have been closed
             self._on_close_ready()
-            return
 
     def remove_timeout(self, callback_method):
         """Adapters should override to call the callback after the
@@ -783,8 +786,10 @@ class Connection(object):
                     self._channels[channel_number].close(reply_code, reply_text)
                 else:
                     del self._channels[channel_number]
-                # Force any lingering callbacks to be removed
-                self.callbacks.cleanup(channel_number)
+                    # Force any lingering callbacks to be removed
+                    # moved inside else block since _on_channel_closeok removes
+                    # callbacks
+                    self.callbacks.cleanup(channel_number)
         else:
             self._channels = dict()
 
@@ -905,7 +910,7 @@ class Connection(object):
         :rtype: bool
 
         """
-        return bool(self._channels)
+        return any([self._channels[num].is_open for num in self._channels])
 
     def _has_pending_callbacks(self, value):
         """Return true if there are any callbacks pending for the specified
@@ -1016,8 +1021,8 @@ class Connection(object):
         except KeyError:
             LOGGER.error('Channel %r not in channels',
                          method_frame.channel_number)
-        # Force any callbacks to be removed for this channel
-        self.callbacks.cleanup(method_frame.channel_number)
+        if self.is_closing and not self._has_open_channels:
+            self._on_close_ready()
 
     def _on_close_ready(self):
         """Called when the Connection is in a state that it can close after
@@ -1053,17 +1058,13 @@ class Connection(object):
         if method_frame and self._is_connection_close_frame(method_frame):
             self.closing = (method_frame.method.reply_code,
                             method_frame.method.reply_text)
-        LOGGER.warning("Disconnected from RabbitMQ at %s:%i (%s): %s",
-                        self.params.host, self.params.port,
-                        self.closing[0], self.closing[1])
+
+        # If this did not come from the connection adapter, close the socket
         if not from_adapter:
             self._adapter_disconnect()
-        self._set_connection_state(self.CONNECTION_CLOSED)
-        if method_frame:
-            for channel in self._channels:
-                self._channels[channel]._on_close(method_frame)
-        self._process_connection_closed_callbacks()
-        self._remove_connection_callbacks()
+
+        # Invoke a method frame neutral close
+        self._on_disconnect(self.closing[0], self.closing[1])
 
     def _on_connection_open(self, method_frame):
         """
@@ -1144,6 +1145,26 @@ class Connection(object):
             self._trim_frame_buffer(consumed_count)
             self._process_frame(frame_value)
 
+    def _on_disconnect(self, reply_code, reply_text):
+        """Invoke passing in the reply_code and reply_text from internal
+        methods to the adapter. Called from on_connection_closed and Heartbeat
+        timeouts.
+
+        :param str reply_code: The numeric close code
+        :param str reply_text: The text close reason
+
+        """
+        LOGGER.warning("Disconnected from RabbitMQ at %s:%i (%s): %s",
+                       self.params.host, self.params.port,
+                       reply_code, reply_text)
+        self._set_connection_state(self.CONNECTION_CLOSED)
+        for channel in self._channels:
+            method_frame = frame.Method(channel, spec.Channel.Close(reply_code,
+                                                                    reply_text))
+            self._channels[channel]._on_close(method_frame)
+        self._process_connection_closed_callbacks(reply_code, reply_text)
+        self._remove_connection_callbacks()
+
     def _process_callbacks(self, frame_value):
         """Process the callbacks for the frame if the frame is a method frame
         and if it has any callbacks pending.
@@ -1161,12 +1182,16 @@ class Connection(object):
             return True
         return False
 
-    def _process_connection_closed_callbacks(self):
+    def _process_connection_closed_callbacks(self, reason_code, reason_text):
         """Process any callbacks that should be called when the connection is
         closed.
 
+        :param str reason_code: The numeric code from RabbitMQ for the close
+        :param str reason_text: The text reason fro closing
+
         """
-        self.callbacks.process(0, '_on_connection_closed', self, self)
+        self.callbacks.process(0, '_on_connection_closed', self, self,
+                               reason_code, reason_text)
 
     def _process_frame(self, frame_value):
         """Process an inbound frame from the socket.
